/* ===================================================
 * Artekit AK-BMG160 Breakout Board.
 *
 * This is a very simple program to demonstrate the 
 * AK-BMG160 3-axis digital gyro.
 * The program can be used with either SPI or I2C by
 * defining the right constant (BMG160_SPI or BMG160_I2C).
 *
 * http://www.artekit.eu/products/breakout-boards/ak-bmg160-3-axis-digital-gyroscope/
 *
 * When SPI is selected:
 *   SPI SCK on pin 13
 *   SPI MISO on pin 12
 *   SPI MOSI on pin 11
 *   SPI SS on pin 10
 * When I2C is selected:
 *   I2C SDA on pin A4
 *   I2C SCL on pin A5
 * Interrupts:
 *   INT1 on pin 2
 *   INT2 on pin 3
 * =================================================== */

#include <Wire.h>
#include <SPI.h>


/* Define here which communication channel to use */
//#define BMG160_SPI
#define BMG160_I2C

#define BMG160_ADDR  0x69

/*
 * Communication functions when SPI is selected
 */
#ifdef BMG160_SPI
static byte ReadRegister(byte reg)
{
  byte ret;
  
  digitalWrite(10, 0);
  SPI.transfer(reg | 0x80);
  ret = SPI.transfer(0x00);  
  digitalWrite(10, 1);
  
  return ret;
}


static void WriteRegister(byte reg, byte value)
{
  digitalWrite(10, 0);
  SPI.transfer(reg & 0x7F);
  SPI.transfer(value);  
  digitalWrite(10, 1);
}


static void ReadData(byte start, byte* data, int len)
{
  int offs = 0;
  
  digitalWrite(10, 0);
  SPI.transfer(start | 0x80);
  
  while (len)
  {
    data[offs++] = SPI.transfer(0x00);  
    len--;
  }
  
  digitalWrite(10, 1);
}

#endif /* BMG160_SPI */

/*
 * Communication functions when I2C is selected
 */
#ifdef BMG160_I2C
/* This function reads a byte for the given register */
static byte ReadRegister(byte reg)
{
  Wire.beginTransmission(BMG160_ADDR);
  Wire.write(reg);
  /* By setting false here we tell the Wire library to
   * repeat the start condition. May not work on older
   * Arduino libraries.
   */
  Wire.endTransmission(false);
  Wire.requestFrom(BMG160_ADDR, 1);
  return Wire.read();
}

/* This function writes a byte to the given register */

static void WriteRegister(byte reg, byte value)
{
  Wire.beginTransmission(BMG160_ADDR);
  Wire.write(reg);
  Wire.write(value);
  Wire.endTransmission(true);
}

static void ReadData(byte start, byte* data, int len)
{
  int offs = 0;
  
  Wire.beginTransmission(BMG160_ADDR);
  Wire.write(start);
  /* By setting false here we tell the Wire library to
   * repeat the start condition. May not work on older
   * Arduino libraries.
   */
  Wire.endTransmission(false);
  Wire.requestFrom(BMG160_ADDR, len);
  while (len)
  {
     data[offs++] = Wire.read();
     len--;
  }
}
#endif /* BMG160_I2C */

void do_test()
{
  byte test;
  
  /* Read chip id */
  test = ReadRegister(0x00);
  Serial.print("Chip ID= ");
  Serial.println(test);
  
  test = ReadRegister(0x3C);
  test |= 0x01;
  WriteRegister(0x3c, test);
  
  do
  {
    test = ReadRegister(0x3c);
    Serial.print(".");
  } while ( (test & 0x02) == 0);
  
  Serial.println("");
  Serial.print("Test ended. Result: ");
  if (test & 0x04)
  {
    Serial.println("Failed"); 
  } else {
    Serial.println("Test OK"); 
  }
}

void config()
{
  int val;
  
  /* Reset any compensation */
  val = ReadRegister(0x21);
  val |= 0x40;
  WriteRegister(0x21, val);
  
  /* Configure full scale */
  WriteRegister(0x0F, 0x80);
  
  /* Bandwidth 100 khz */
  WriteRegister(0x10, 4);
  
  /* Filtered data, shadow registers enabled */
  WriteRegister(0x13, 0);
  
  /* New data interrupt enable */
  WriteRegister(0x15, 0x80);
  
  /* Push-pull, active high interrupt pins */
  WriteRegister(0x16, 5);
  
  /* 'any-motion' interrupt on int1 pin */
  WriteRegister(0x17, 2);
  
  /* New data interrupt on int2 pin */
  WriteRegister(0x18, 0x80);
  WriteRegister(0x19, 0);
  
  /* Remove interrupt selectable data source */
  WriteRegister(0x1A, 0);
  
  /* 9,7 slope any-motion threshold (16 samples duration
   * configured in register 0x1C)
   * 4000 d/s / 65535 = 0,061 LSB
   * any_th = (1 + reg_value) * 16 LSB
   * 9,7 = (1 + 9) * 0,97
   */
  WriteRegister(0x1B, 9);

  /* Enable on all axis on a minimum movement and keep the int1 pin
   * asserted as long as possible
   */
  WriteRegister(0x1C, 0x37);
  
  /* Interrupt mode latched */
  WriteRegister(0x21, 7);
  
  /* Slow offset cancellation 1 degree/s over 40ms */
  WriteRegister(0x31, 0xC7);  
}

void read_and_print()
{
  byte data[6];
  signed short* values = (signed short*) data;
  
  ReadData(0x02, data, 6);

  Serial.print("X = ");  
  Serial.print(values[0] * 0.06103f); 
    
  Serial.print(" Y = ");
  Serial.print(values[1] * 0.06103f);
   
  Serial.print(" Z = ");
  Serial.println(values[2] * 0.06103f);
  
}

void setup()
{
  #ifdef BMG160_I2C
  Wire.begin();
  #endif /* USE_I2C */
  
  #ifdef BMG160_SPI
  SPI.begin();
  SPI.setBitOrder(MSBFIRST);
  SPI.setDataMode(SPI_MODE0);
  
  /* Chip select */
  pinMode(10, OUTPUT);
  digitalWrite(10, 1);
  #endif /* USE_SPI */
  
  Serial.begin(9600);
  
  /* int1 on D2, int2 on D3*/
  pinMode(2, INPUT);
  pinMode(3, INPUT);
  
  do_test();
  config();
}

void loop()
{
    /* Any-motion interrupt */
    if (digitalRead(2))
    {
      Serial.println("Any-Motion detected"); 
      /* Clear interrupt */
      WriteRegister(0x21, 0x87);  
    }

    /* Check on new data */
    if (digitalRead(3))
    {
      read_and_print();
    }
}

